% Nonlinear CSTR  model

% 9 states
% 4 inputs
% 3 disturbances

% Initial values:
% X_Glu = 0; % g/L
% X_Xyl = 0; % g/L
% Glu = 20; % g/L
% Man = 20; % g/L
% Xyl = 20; % g/L
% P = 0; % M
% V = 0.7 % L
% V_RiboGlu = 1 % L
% V_RiboXyl = 1 % L

% Constraints
% % For x = [X_RiboGlu, X_RiboXyl, Glu, Man, Xyl, P , V , V_RiboGlu, V_RiboXyl]
% x_lower = [0 0 0 0 0 0 0.7 0.7 0.7 ]; %
% script not to run
% x_upper = [ inf, inf, inf, inf, inf, inf, inf 1 1 1 1];
%
% % For u = [F_RiboGlu_in,  F_RiboXyl_in, , F_SSL_in, F_out] % 
% u_lower = [0, 0, 0, 0]; 
% u_upper = [ 1, 1, 1, 1]; 
%
function ODEsys = ODEmodel()
%addpath ('/users/dinark/Downloads/casadi-osx-matlabR2015a-v3.5.1/')
import casadi.*
ODEsys = struct; %Empty struct

% States, declaring them symbolically
X_RiboGlu = SX.sym('X_RiboGlu'); %Biomass, g/L
X_RiboXyl = SX.sym('X_RiboXyl'); %Biomass, g/L
Glu = SX.sym('Glu'); %Substrate, g/L
Man = SX.sym('Man'); %Substrate, g/L
Xyl = SX.sym('Xyl'); %Substrate, g/L
P= SX.sym('P'); % Riboflavin, M
V=SX.sym('V'); % Reactor volume, L
V_RiboGlu=SX.sym('V_RiboGlu'); %RiboGlu reactor volume, L
V_RiboXyl=SX.sym('V_RiboXyl');%RiboXyl reactor volume, L
%Filling up the ODEsys struct
ODEsys.x = [X_RiboGlu, X_RiboXyl, Glu, Man, Xyl, P, V, V_RiboGlu,  V_RiboXyl]';
ODEsys.x_names = ['X_RiboGlu',  'X_RiboXyl', 'Glu', 'Man', 'Xyl', 'P', 'V', 'V_RiboGlu', 'V_RiboXyl']';

% Inputs - Normalized by reactor volume.
F_RiboGlu_in=SX.sym('F_RiboGlu_in'); %L/h
F_RiboXyl_in=SX.sym('F_RiboXyl_in');  %L/h 
F_SSL_in=SX.sym('F_SSL_in'); %L/h - sugar stream
F_out= SX.sym('F_out');%L/h

ODEsys.u = [F_RiboGlu_in, F_RiboXyl_in, F_SSL_in, F_out]';
ODEsys.u_names = [F_RiboGlu_in, F_RiboXyl_in, F_SSL_in, F_out]';

% Uncertainties
% SSL feed in stream concentrations
SSL_Glu_in = SX.sym('SSL_Glu_in'); % g/L
SSL_Man_in = SX.sym('SSL_Man_in'); % g/L
SSL_Xyl_in = SX.sym('SSL_Xyl_in'); % g/L
ODEsys.d = [SSL_Glu_in,SSL_Man_in,SSL_Xyl_in]';
ODEsys.d0 = [36,110,45]'; % g/L 

%Substrate concentrations
%Assuming the S_in are zero, as they come from a completed batch with
%sugars consumed.
%Sugars from RiboGlu strain
RiboGlu_Glu_in=0;
RiboGlu_Man_in=0;
RiboGlu_Xyl_in=0;
%Sugars from RiboXyl strain
RiboXyl_Glu_in=0;
RiboXyl_Man_in=0;
RiboXyl_Xyl_in=0;

%Biomass concentrations
%Assuming the fermentations are prepared so that all strains have the same
%concentration once MPC is started 
%RiboGlu
RiboGlu_X_RiboGlu_in=7;
RiboGlu_X_RiboMan_in=0;
RiboGlu_X_RiboXyl_in=0;
%RiboXyl
RiboXyl_X_RiboGlu_in=0;
RiboXyl_X_RiboMan_in=0;
RiboXyl_X_RiboXyl_in=7;


%Riboflavin concentrations 
%Interested in the riboflavin produced in main reactor
RiboGlu_P_in=0;
RiboXyl_P_in=0;
SSL_P_in=0;


%% #####################################################
%                       Parameters
%              Obtained from Parameter_Esitmation_Strain

%% RiboGlu
mumax1_RiboGlu=0.1290;
mumax2_RiboGlu=0.7299;
mumax3_RiboGlu=0.0013;
Ks1_RiboGlu=0.0523;
Ks2_RiboGlu=523.9684;
Ks3_RiboGlu=834.3772;
Y_xs1_RiboGlu=-0.3806;
Y_xs2_RiboGlu=-0.1716;
Y_xs3_RiboGlu=-0.0050;
Y_px_RiboGlu=0.7663;


% #######################################
%               Constants
% #######################################

mu1_RiboGlu= mumax1_RiboGlu*Glu/(Ks1_RiboGlu+Glu);
mu2_RiboGlu = mumax2_RiboGlu*Man/(Ks2_RiboGlu+Man);
mu3_RiboGlu = mumax3_RiboGlu*Xyl/(Ks3_RiboGlu+Xyl);
rx1_RiboGlu = mu1_RiboGlu*X_RiboGlu;
rx2_RiboGlu = mu2_RiboGlu*X_RiboGlu;
rx3_RiboGlu = mu3_RiboGlu*X_RiboGlu;
rx_RiboGlu = rx1_RiboGlu + rx2_RiboGlu + rx3_RiboGlu;
rs1_RiboGlu = rx1_RiboGlu/Y_xs1_RiboGlu;
rs2_RiboGlu = rx2_RiboGlu/Y_xs2_RiboGlu;
rs3_RiboGlu = rx3_RiboGlu/Y_xs3_RiboGlu;
rp_RiboGlu=Y_px_RiboGlu*rx_RiboGlu;

%% RiboXyl
mumax1_RiboXyl=0.0675;
mumax2_RiboXyl=0.0003;
mumax3_RiboXyl=0.1082;
Ks1_RiboXyl=0.0905;
Ks2_RiboXyl=653.7131;
Ks3_RiboXyl=2.0783;
Y_xs1_RiboXyl=-0.2396;
Y_xs2_RiboXyl=-0.0001;
Y_xs3_RiboXyl=-0.2876;
Y_px_RiboXyl=0.9751;


% #######################################
%               Constants
% #######################################

mu1_RiboXyl= mumax1_RiboXyl*Glu/(Ks1_RiboXyl+Glu);
mu2_RiboXyl = mumax2_RiboXyl*Man/(Ks2_RiboXyl+Man);
mu3_RiboXyl = mumax3_RiboXyl*Xyl/(Ks3_RiboXyl+Xyl);
rx1_RiboXyl = mu1_RiboXyl*X_RiboXyl;
rx2_RiboXyl = mu2_RiboXyl*X_RiboXyl;
rx3_RiboXyl = mu3_RiboXyl*X_RiboXyl;
rx_RiboXyl = rx1_RiboXyl + rx2_RiboXyl + rx3_RiboXyl;
rs1_RiboXyl = rx1_RiboXyl/Y_xs1_RiboXyl;
rs2_RiboXyl= rx2_RiboXyl/Y_xs2_RiboXyl;
rs3_RiboXyl = rx3_RiboXyl/Y_xs3_RiboXyl;
rp_RiboXyl=Y_px_RiboXyl*rx_RiboXyl;


%% Initial values of states

% Initial Values of states
ODEsys.x0 = [0,0, 20, 20, 20, 0, 0.7, 1, 1]';
ODEsys.u0 = [0,0,0,0,0]';

% For x = [X_RiboGlu, X_RiboMan, X_RiboXyl, Glu, Man, Xyl, P, V, V_RiboGlu, V_RiboMan, V_RiboXyl]
ODEsys.x_lower = [0, 0,0,0,0,0 0.7, 0.7, 0.7]'; 
ODEsys.x_upper = [  inf, inf, inf, inf, inf, inf, 1, 1, 1]';

% For u = [F_RiboGlu_in, F_RiboMan_in, F_RiboXyl_in, F_SSL_in, F_out] % 
     ODEsys.u_lower = [0,0, 0,0]';
     ODEsys.u_upper = [1, 1, 1, 1]';
     %% Constant F_SSL_in of 0.1 L/h
% ODEsys.u_lower = [0,0,0, 0.1,0]'; 
% ODEsys.u_upper = [1, 1, 1, 0.1, 1]';

ODEsys.delta_U_lower = [ -inf, -inf, -inf,-inf]';
ODEsys.delta_U_upper = [ inf, inf, inf, inf]';

%% Dilutions - change of concentrations because of streams in and out
%Sugars
D_Glu= F_SSL_in*(SSL_Glu_in-Glu)/V + F_RiboGlu_in*(RiboGlu_Glu_in-Glu)/V + ...
     F_RiboXyl_in*(RiboXyl_Glu_in-Glu)/V ;
D_Man=F_SSL_in*(SSL_Man_in-Man)/V + F_RiboGlu_in*(RiboGlu_Man_in-Man)/V + ...
     F_RiboXyl_in*(RiboXyl_Man_in-Man)/V;
D_Xyl=F_SSL_in*(SSL_Xyl_in-Xyl)/V + F_RiboGlu_in*(RiboGlu_Xyl_in-Xyl)/V + ...
     F_RiboXyl_in*(RiboXyl_Xyl_in-Xyl)/V;
%Biomass
D_RiboGlu= F_SSL_in*(0-X_RiboGlu)/V+ ...
    F_RiboGlu_in*(RiboGlu_X_RiboGlu_in-X_RiboGlu)/V + ...
     F_RiboXyl_in*(RiboXyl_X_RiboGlu_in-X_RiboGlu)/V ;
D_RiboXyl= F_SSL_in*(0-X_RiboXyl)/V+ ...
     F_RiboGlu_in*(RiboGlu_X_RiboXyl_in-X_RiboXyl)/V + ...
    F_RiboXyl_in*(RiboXyl_X_RiboXyl_in-X_RiboXyl)/V;
%Riboflavin
D_P= F_SSL_in*(SSL_P_in-P)/V+ ...
     F_RiboGlu_in*(RiboGlu_P_in-P)/V + ...
    F_RiboXyl_in*(RiboXyl_P_in-P)/V;

%%  ODEs
X_RiboGlu_dot = rx_RiboGlu + D_RiboGlu; 
X_RiboXyl_dot= rx_RiboXyl + D_RiboXyl; 
Glu_dot = rs1_RiboGlu +  rs1_RiboXyl  + D_Glu;
Man_dot = rs2_RiboGlu + rs2_RiboXyl +  D_Man;
Xyl_dot= rs3_RiboGlu + rs3_RiboXyl + D_Xyl;
P_dot = rp_RiboGlu + rp_RiboXyl + D_P ;
V_dot=F_RiboGlu_in +F_RiboXyl_in + F_SSL_in - F_out; 
V_RiboGlu_dot=-F_RiboGlu_in;
V_RiboXyl_dot=-F_RiboXyl_in;
ODEsys.x_dot = [X_RiboGlu_dot, X_RiboXyl_dot, Glu_dot, Man_dot, Xyl_dot, P_dot, V_dot, V_RiboGlu_dot, V_RiboXyl_dot]'; % endre

end
