function plotFunction2(x0,sim,ODEmodel,OCPprops)

x_ul = ODEmodel.x_upper;
x_ll = ODEmodel.x_lower;
u_ul = ODEmodel.u_lower;
u_ll = ODEmodel.u_upper;

%Get it in hours instead of timesteps
i=length(sim.y);
T=OCPprops.T;
N=OCPprops.N;
h=T/N;
xaxis=[0:h:i*h];

%accumulated riboflavin molar flow (M/h)
acc=[]; 
for j = 1:i 
    
    if j==1
        acc(j)= sim.y(7,j)*sim.u(5,j);
    else
        acc(j)=acc(j-1) + (sim.y(7,j)*sim.u(5,j));
    end
end    
acc=acc*h; %To get M unit


b = figure();
subplot(5,1,1)
stairs(xaxis,[sim.u(1,:), NaN])
ylabel({'F_{RiboGlu}_{,in}';'[L/h]'})
ylim([0 0.75])
xlabel('Time [h]')
subplot(5,1,2)
stairs(xaxis,[sim.u(2,:), NaN])
title('')
ylabel({'F_{RiboMan}_{,in}';'[L/h]'})
ylim([0 0.75])
xlabel('Time [h]')
subplot(5,1,3)
stairs(xaxis,[sim.u(3,:), NaN])
title('')
ylabel({'F_{RiboXyl}_{,in} ';'[L/h]'})
ylim([0 0.75])
xlabel('Time [h]')
subplot(5,1,4)
stairs(xaxis,[sim.u(4,:), NaN])
title('')
ylabel({'F_{Synthetic SSL}_{,in}';'[L/h]'})
ylim([0 1])
xlabel('Time [h]')
subplot(5,1,5)
stairs(xaxis,[sim.u(5,:), NaN])
title('')
ylabel({'F_{out}';'[L/h]'})
ylim([0 1])
editFigureProperties(b);
xlabel('Time [h]')
saveas(gcf,'Inputs.png')
saveas(gcf,'Inputs.fig')

c = figure();
subplot(2,1,1)
hold on
plot(xaxis,[x0(4), sim.y(4,:)])
plot(xaxis,[x0(5), sim.y(5,:)])
plot(xaxis,[x0(6), sim.y(6,:)])
legend('Glucose', 'Mannose', 'Xylose')
legend('Location','northeastoutside')
xlabel('Time [h]')
ylabel('Sugar concentration [g/L]')
ylim([0 120])
subplot(2,1,2)
hold on
plot(xaxis,[x0(1), sim.y(1,:)])
plot(xaxis,[x0(2), sim.y(2,:)])
plot(xaxis,[x0(3), sim.y(3,:)])
legend('RiboGlu', 'RiboMan', 'RiboXyl')
legend('Location','northeastoutside')
xlabel('Time [h]')
ylabel('Biomass concentration [g/L]')
ylim([0 20])
editFigureProperties(c);
saveas(gcf,'States.png')
saveas(gcf,'States.fig')

d= figure();
subplot(4,1,1)
hold on
plot(xaxis,[x0(8), sim.y(8,:)])
ylabel('V_{Main reactor} [L]')
ylim([0 1])
xlabel('Time [h]')
subplot(4,1,2)
hold on
plot(xaxis,[x0(9), sim.y(9,:)])
ylabel('V_{RiboGlu} [L]')
ylim([0 1])
xlabel('Time [h]')
subplot(4,1,3)
hold on
plot(xaxis,[x0(10), sim.y(10,:)])
ylabel('V_{RiboMan} [L]')
ylim([0 1])
xlabel('Time [h]')
subplot(4,1,4)
hold on
plot(xaxis,[x0(11), sim.y(11,:)])
ylabel('V_{RiboXyl} [L]')
ylim([0 1])
xlabel('Time [h]')
editFigureProperties(d);
saveas(gcf,'Volumes.png')
saveas(gcf,'Volumes.fig')

e= figure();
subplot(3,1,1)
plot(xaxis, [x0(7),sim.y(7,:)])
ylabel({'Riboflavin' ;'concentration in reactor [M]'})
ylim([0 30])
xlabel('Time [h]')
subplot(3,1,2)
plot(xaxis, [x0(7),sim.y(7,:).*sim.u(5,:)])
ylabel({'Riboflavin';' in F_{out} [mol/h]'})
ylim([0 20])
xlabel('Time [h]')
subplot(3,1,3)
plot(xaxis,[x0(7), acc])
ylabel({'Riboflavin ';'accumulated in F_{out} [mol]'})
ylim([0 300])
xlabel('Time [h]')
editFigureProperties(e);
saveas(gcf,'Riboflavin.png')
saveas(gcf,'Riboflavin.fig')


end