%% RiboMan with multiple sugars
function results =ParameterEstimation_RiboMan() 
close all
% Estimation of ODE parameters by non-linear least squares optimization 
% solver. 
% Estimate parameters, such as the residuals are minimum. The variables in 
% simulations will be as close as possible to the response in experiments. 
%
%All you need is an initial guess for each of the parameter(s) you want to
%estimate. 
% the data was generated by running PosFeed_Exp_generator()
% -------------------------------------------------------------
%% 
    % here is the initial guesses of the parameters    
initParams=[0.0898    4.5497    0.0000  934.4947   -0.4378   -0.1395    1.9953];% estimating 7 parameters

%0.2933   20.0000   -0.4113    1.0000
%exper=[0 0]; tspan=0;
%% Known values
%correction factor of the lab
cfac=0.3634;
%molarmasses g/mmol
M_glu=0.18;
M_man=0.18;
M_xyl=0.15013;
%%
%defining
x0 = 0;
tspan = 0;
%%        Glucose
%###########################
%exper=[0 0]; tspan=0;
t_end=34; %The time when batch phase is over
rawdata = readtable('RiboMan_Glu.xlsx');

for i = 1:width(rawdata)
    if iscell(rawdata{:,i})
        data(:,i) = str2double(rawdata{:,i});
    else
        data(:,i) = rawdata{:,i};
    end
end

%% Finding slopes for S and P
Glu_peak = data(2,7:10);
Glu_conc = data(1,7:10);
Glu_slope=polyfit(Glu_conc,Glu_peak,1);
Glu_slope=Glu_slope(1);

Ribo_peak = data(2,11:14);
Ribo_conc = data(1,11:14);
Ribo_slope=polyfit(Ribo_conc,Ribo_peak,1);
Ribo_slope=Ribo_slope(1);

% The first values of each state
time_data = data(:,1); % Time column in data sheet
timespan{1} = time_data(time_data<t_end)';
X_exp = data(1:length(timespan{1}),2)*cfac; %concentration [g/L]
Glu_exp = (data(1:length(timespan{1}),3)/Glu_slope*10)*M_glu; %concentration [g/L]
Man_exp= zeros(size(Glu_exp));
P_exp = (data(1:length(timespan{1}),4)/Ribo_slope*10)*1000; %concentration [M]
x_init(1,:) = [X_exp(1),Glu_exp(1), Man_exp(1), P_exp(1)];
exper{1} = [X_exp, Glu_exp, Man_exp, P_exp];
opts = optimset('TolFun', 1e-12, 'TolX', 1e-12, 'MaxIter', 1000, 'Diagnostics', 'off', 'Display', 'iter');

%%      Mannose
%###########################
t_end2=68; %The time when batch phase is over
rawdata2 = readtable('RiboMan_Man.xlsx');
for i = 1:width(rawdata2)
    if iscell(rawdata2{:,i})
        data2(:,i) = str2double(rawdata2{:,i});
    else
        data2(:,i) = rawdata2{:,i};
    end
end
%% Finding slopes for S and P
Man_peak = data2(2,7:10);
Man_conc = data2(1,7:10);
Man_slope=polyfit(Man_conc,Man_peak,1);
Man_slope=Man_slope(1);

Ribo_peak2 = data2(2,11:13);
Ribo_conc2 = data2(1,11:13);
Ribo_slope2=polyfit(Ribo_conc2,Ribo_peak2,1);
Ribo_slope2=Ribo_slope2(1);

% The first values of each state
time_data = data2(:,1); % Time column in data sheet
timespan{2} = time_data(time_data<t_end2)';
X_exp = data2(1:length(timespan{2}),2)*cfac; %concentration [g/L]
Man_exp = (data2(1:length(timespan{2}),3)/Glu_slope*10)*M_man; %concentration [g/L]
Glu_exp = zeros(size(Man_exp));
P_exp = (data2(1:length(timespan{2}),4)/Ribo_slope2*10)*1000; %concentration [M]
x_init(2,:) = [X_exp(1),Glu_exp(1), Man_exp(1),P_exp(1)]; %initial states
exper{2} = [X_exp,Glu_exp, Man_exp, P_exp]; %experimental
opts = optimset('TolFun', 1e-12, 'TolX', 1e-12, 'MaxIter', 1000, 'Diagnostics', 'off', 'Display', 'iter');

%% HERE you need to define your bounds for each of your parameter
% Give a lower and an upper limit for each parameter

loBound = [0.0 0.0 0.0 0.0 -1 -1 0.0];
upBound = [20 20 1000 1000 0 0 100]; 

res = lsqnonlin(@residual,  initParams, loBound, upBound, opts);

%results

% Plotting
%Glucose plot
figure(1)
plot(timespan{1},Y{1})
hold on
plot(timespan{1},exper{1}(:,1),'*')
plot(timespan{1},exper{1}(:,2),'*')
plot(timespan{1},exper{1}(:,4),'*')
legend('X [g/L]','Glu [g/L]', 'Man [g/L]' ,'P [M]','X_{exp} [g/L]','Glu_{exp} [g/L]','P_{exp} [M]')
title('RiboMan strain with glucose as carbon source')
xlabel('Time [h]')
ylabel('Concentration')

%Mannose plot
figure(2)
plot(timespan{2},Y{2})
hold on
plot(timespan{2},exper{2}(:,1),'*')
plot(timespan{2},exper{2}(:,3),'*')
plot(timespan{2},exper{2}(:,4),'*')
legend('X [g/L]','Glu [g/L]', 'Man [g/L]' ,'P [M]','X_{exp} [g/L]','Man_{exp} [g/L]','P_{exp} [M]')
title('RiboMan strain with mannose as carbon source')
xlabel('Time [h]')
ylabel('Concentration')
%initParams=b (endres underveis, men frst er den det)
   function R = residual(b)
        
       a = b;
       
       R = [];
       Y = [];
       
       for j = 1:2
           tspan = timespan{j};
           x0 = x_init(j,:);
       [T ,Y{j} ]= modelsolve (a);
        R = [R; exper{j} - Y{j}];
       end
       
       
      %residual = exper(:,2) - Y(:,2);
     
       %R = residual(:);
       R(isnan(R)) = 0;
       R(:,1) = 10*R(:,1); %Biomass is weighted more
       
       results=a;
       

   end

function [ T, Y ] = modelsolve( a)
%% HERE you define all the parameter i you wish to estimate as a(i):
% if you comment here, you must update the ODE below (In function REACTION( t,x) below.).

mumax1=a(1);
mumax2= a(2);
Ks1=a(3);
Ks2=a(4);
Y_xs1=a(5);
Y_xs2=a(6);
Y_px=a(7);


%% Solve ode

[T,Y] = ode45(@model, tspan, x0, []);

function x_dot = model(t, x)

% Unpack initial values    
%Act=x(1);
%yP=x(2);
X=x(1);
Glu=x(2);
Man=x(3); % Blir dette riktig n? 
P=x(4);
%% HERE you need to remove the parameters that you estimate, and leave the known (fixed) ones. 

% Define all known constants
mu1= mumax1*Glu/(Ks1+Glu);
mu2 = mumax2*Man/(Ks2+Man);
rx1 = mu1*X;
rx2 = mu2*X;
rx = rx1 + rx2;
rs1 = rx1/Y_xs1;
rs2 = rx2/Y_xs2;
rp=Y_px*rx;

%Diff eq
X_dot = rx;
Glu_dot = rs1;
Man_dot = rs2;
P_dot = rp;

x_dot = [X_dot, Glu_dot, Man_dot, P_dot]';


    
end

end

end


