%% Without maintenance, Glucose, without product
%RiboMan
function results =ParameterEstimation_RiboMan_Glu() 

% Estimation of ODE parameters by non-linear least squares optimization 
% solver. 
% we do not know the parameters themselves. We want to estimate thsee, such as the
% residuals are minimum (see lecture notes), that is, the variables in
% simulations will be as close as possible to the response in experiments. 
%
%All you need is an initial guess for each of the parameter(s) you want to
%estimate. 

% the data was generated by running PosFeed_Exp_generator()
% -------------------------------------------------------------
%% 
    % here is the initial guesses of the parameters
      
initParams=[ 0.1213    0.0072   -0.3350  ];% estimating 3 parameters

%exper=[0 0]; tspan=0;
t_end=34; %The time when batch phase is over
rawdata = readtable('RiboMan_Glu.xlsx');
%correction factor of the lab
cfac=0.3634;
%molarmasses g/mmol
M_glu=0.18;
M_man=0.18;
M_xyl=0.15013;

for i = 1:width(rawdata)
    if iscell(rawdata{:,i})
        data(:,i) = str2double(rawdata{:,i});
    else
        data(:,i) = rawdata{:,i};
    end
end

%% Finding slopes for S and P
Glu_peak = data(2,7:10);
Glu_conc = data(1,7:10);
Glu_slope=polyfit(Glu_conc,Glu_peak,1);
Glu_slope=Glu_slope(1);

Ribo_peak = data(2,11:14);
Ribo_conc = data(1,11:14);
Ribo_slope=polyfit(Ribo_conc,Ribo_peak,1);
Ribo_slope=Ribo_slope(1);

% The first values of each state
tspan = data(:,1); % Time column in data sheet
tspan = tspan(tspan<t_end);
X_exp = data(1:length(tspan),2)*cfac; %concentration [g/L]
S1_exp = (data(1:length(tspan),3)/Glu_slope*10)*M_glu; %concentration [g/L]
%P_exp = (data(1:length(tspan),4)/Ribo_slope*10)*1000; %concentration [M]
x0 = [X_exp(1),S1_exp(1)];
exper = [X_exp, S1_exp];
opts = optimset('TolFun', 1e-12, 'TolX', 1e-12, 'MaxIter', 1000, 'Diagnostics', 'off', 'Display', 'iter');

%% HERE you need to define your bounds for each of your parameter
% Give a lower and an upper limit for each parameter

loBound = [0.0 0.0 -1 0.0];
upBound = [20 1000 0 1]; 

res = lsqnonlin(@residual,  initParams, loBound, upBound, opts);

% Plotting
figure()
plot(tspan,Y)
hold on
plot(tspan,X_exp,'*')
plot(tspan,S1_exp,'*')
legend('X','S','X_{exp}','S_{exp}')

%initParams=b (endres underveis, men frst er den det)
   function R = residual(b)
        
       a = b;
         
       [T ,Y ]= modelsolve (a);
       
       R = exper - Y;
      %residual = exper(:,2) - Y(:,2);
     
       R(isnan(R)) = 0;
       
       results=a;
       

   end

function [ T, Y ] = modelsolve( a )
%% HERE you define all the parameter i you wish to estimate as a(i):
% if you comment here, you must update the ODE below (In function REACTION( t,x) below.).
mumax=a(1);
Ks=a(2);
Y_xs=a(3);
%Y_px=a(4);
%mp=a(5)


%% Solve ode

[T,Y] = ode45(@model, tspan, x0, []);

function x_dot = model(t, x)

% Unpack initial values   
X=x(1)
S1=x(2)
%P=x(3)
%% HERE you need to remove the parameters that you estimate, and leave the known (fixed) ones. 

% Define all known constants
mu= mumax*S1/(Ks+S1);
rx=mu*X;
rs=rx/Y_xs;
%rp=Y_px*rx; %- production without maintenance
%rp=(Y_px*mu+mp)*X

%Diff eq
X_dot = rx;
S1_dot = rs;
%S2_dot = 1
%S3_dot =1
%P_dot = rp;

x_dot = [X_dot, S1_dot]';

end

end

end


